const http = require('http');
const url = require('url');
const path = require('path');
const mime = require("mime");
const fs = require('fs');
const os = require('os');
const zlib = require('zlib');

let port = process.argv[2];

if (!port) {
  port = 8888;
}

http.createServer((req, res) => {
  route(req, res);
}).listen(port, () => {
  console.log(`server listen on port ${port}`);
  let netWorkInterfaces = os.networkInterfaces();
  for (let networkInterface in netWorkInterfaces) {
    for (let el of netWorkInterfaces[networkInterface]) {
      if (el.family == "IPv4" && !el.address.match(/^127./)) {
        console.log(`IP: ${el.address}`);
      }
    }
  }
});

function route(req, res) {
  let reqUrl = decodeURIComponent(req.url);
  let urlObj = url.parse(reqUrl);
  let pathname = urlObj.pathname;
  console.log(pathname);
  let absPath = path.join(`${process.cwd()}`, pathname);
  if (fs.lstat(absPath, (err, stat) => {
    if (!err) {
      if (stat.isDirectory()) {
        fs.readdir(absPath, (err, files) => {
          let fileList = [];
          let dirList = [];
          files.forEach(file => fs.lstat(`${path.join(absPath, file)}`, (err, stat) => {
            let item = `
              <li>
                <a href="${encodeURIComponent(file) + (stat.isDirectory() ? '/' : '')}">${file}</a>
              </li>
            `;
            if (stat.isDirectory()) {
              dirList.push(item);
            } else {
              fileList.push(item);
            }
            if (fileList.concat(dirList).length == files.length) {
              let html = `
              <!DOCTYPE html>
              <html>
                <head>
                  <meta charset="utf-8">
                  <title>${req.url}</title>
                </head>
                <body>
                  ${fileList.length ? '<h2>文件</h2>' : ''}
                  <ul> ${fileList.join('')} </ul>
                  ${dirList.length ? '<h2>目录</h2>' : ''}
                  <ul> ${dirList.join('')} </ul>
                </body>
              </html>
              `;

              compress(req, res, html).then(compressedHtml => {
                res.writeHead(200, {"Content-Type": "text/html"});
                res.end(compressedHtml);
              }, err => res.end(JSON.stringify(err)));
            }
          }));
        });
      } else { // 是一个文件
        absPath = absPath.replace(/[\\\/]*$/, '');
        let rStream = fs.createReadStream(absPath);
        res.writeHead(200, {"Content-Type": `${mime.lookup(absPath)}; charset=utf-8`});
        rStream.on('error', e => console.log(e));
        rStream.on('close', () => res.end());
        rStream.pipe(res);
      }
    } else {
      res.end(JSON.stringify(err));
    }
  })) ;
  }

// deflate、gzip 压缩
function compress(req, res, data) {
  let acceptEncoding = req.headers['accept-encoding'];
  if (!acceptEncoding) {
    acceptEncoding = '';
  }
  return new Promise((resolve, reject) => {
    if (acceptEncoding.match(/\bgzip\b/)) {
      res.setHeader('Content-Encoding', 'gzip');
      zlib.gzip(data, (err, data) => {
        if (!err) {
          resolve(data);
        } else {
          reject(err);
        }
      })
    } else if (acceptEncoding.match(/\bdeflate\b/)) {
      res.setHeader('Content-Encoding', 'deflate');
      zlib.deflate(data, (err, data) => {
        if (!err) {
          resolve(data);
        } else {
          reject(err);
        }
      })
    } else {
      resolve(data);
    }
  })
}
